# -*- coding: utf-8 -*-

from tuikit.container import Container
from tuikit.editbox import EditBox
from tuikit.scrollbar import VScrollbar


class TextEdit(Container):
    def __init__(self, width=20, height=20, text=''):
        Container.__init__(self, width, height)

        self.editbox = EditBox(width-2, height-2, text)
        self.add(self.editbox)
        self.editbox.x = 1
        self.editbox.y = 1
        self.editbox.add_handler('scroll', self.on_editbox_scroll)
        self.editbox.add_handler('areasize', self.on_editbox_areasize)

        self.vscroll = VScrollbar(height - 2)
        self.add(self.vscroll)
        self.vscroll.x = width - 1
        self.vscroll.y = 1
        self.vscroll.add_handler('change', self.on_vscroll_change)

        self.on_editbox_areasize(None)

    def settext(self, text):
        self.editbox.set_text(text)

    def scrolltoend(self):
        self.editbox.move_pagelast()

    def on_draw(self, ev):
        ev.driver.frame(ev.x, ev.y, self.width, self.height)

    def on_editbox_scroll(self, ev):
        self.vscroll.pos = self.editbox.yofs

    def on_editbox_areasize(self, ev):
        smax = len(self.editbox.lines) - self.editbox.height
        if smax < 0:
            smax = 0
        self.vscroll.max = smax

    def on_vscroll_change(self, ev):
        self.editbox.set_yofs(self.vscroll.pos)
        self.editbox.redraw()

